#include "psall.h"

char mod_date[]={"05/26/97"};   /* last modification date */

/*

        psall by T. W. Shield

        plots GF, CP, IMG, CP and SGI files to postscript

        needs
                psall.c (this file)
                psinput.c
                psplot.c

                xinput.c
                gfxps.c
                gfxsubs.c

                icolor.c (from ICOP)

        need to add
                -b switch for batch processing
                SGI file support (done)
                NTSC color maps (done)

        bugs fixed:
                12/9/95 fixed rotating plot
                        allow eps
                        x-y axes width/color choice
                        preserve aspect in zaxis works

                12/10/95 added icolor to make legal NTSC colors

                12/17/95 added SGI file support (needs -limage or -limg)
                         use -DSGI to enable
                01/21/96 fixed SGI file aspect ratio calc
                         flipped sgi file read
                         
                02/12/97 tweaked bounding box -- moved gfxps.c 
                
                02/13/97 zaxis label spacing tweaked -- 
                	  added ps_fheight,ps_fwidth globals
                	  
                05/26/97 update to new xinput routines
                	 uses ../gfx/gfxps.c again, added psbb to psall.c
                	 tweaked bounding box again -- needs to be smarter

*/



main(argc,argv)
int argc;
char **argv;
{

extern int    optind;  /* for command line options parsing */
extern char   *optarg;
char  *cmd;
char  *display = NULL;
int c;
char *geo = NULL;
char *defgeo = "500x400+200+300";  /* default geometry for the graphics window */
int bw = 2;  /* border width, gets ignored by window manager */
int x,y,xwidth,xheight;

  cmd = argv[0];
  while ((c = getopt(argc, argv, "hvd:g:")) != EOF)
    switch(c)   {

    case 'd':
      display = optarg;
      break;
    case 'g':
      geo = optarg;
      break;
    case 'v':
    	debug = 1;
    	break;
    case 'h':
      usage(cmd);
      exit(0);
    }


  if ((mydisplay = XOpenDisplay(display)) == NULL)
  {
    fprintf(stderr, "Can\'t open display: %s\n", display );
    exit(0);
  }

        /* initialization */

        myscreen = DefaultScreen(mydisplay);

        /* initialize input windows using color and font defaults */
        
        xi_init(mydisplay,myscreen,NULL,NULL,NULL);

        XGeometry (mydisplay, myscreen, geo, defgeo, bw, 1, 1, 0, 0,
                        &main_xpos, &main_ypos, &xwidth,&xheight);

        defaults(); /* does input wins init too */

/* get command line file name if there */

    if( optind < argc )
    {
        strcpy(set_filename,argv[optind]);
        x=main_xpos+inputwin[MAINWIN].xsize/2;
        y=main_ypos+inputwin[MAINWIN].ysize/2;
        read_settings(x,y);
    }

        main_form(&main_xpos,&main_ypos);
}

get_gf(int x,int y)
{
float xmax2,xmin2,ymax2,ymin2,xtick2,ytick2;
int s1_2,s2_2;
char xlabel2[50],ylabel2[50],ticks2;

      if(strlen(gf_filename) > 0)
      {
         if(read_set == 1)
         {
         /* if used settings file, save the scaling */
                xmax2=xmax;
                xmin2=xmin;
                ymax2=ymax;
                ymin2=ymin;
                ticks2=ticks;
                s1_2=s1;
                s2_2=s2;
                xtick2=xtick;
                ytick2=ytick;
                strcpy(xlabel2,xlabel);
                strcpy(ylabel2,ylabel);
         }

         if(read_gf(gf_filename,x,y) != -1)
         {
              sprintf(temp,"GF file read with %i points",np);
              info_window(x,y,temp);
              plot_gf = 1;
              strcpy(filecurrent,gf_filename);
              if(read_set == 0)
              {
                  plot_lists_defaults(PLOTWIN);
              }
              else
              {
         /* if used settings file, restore the scaling */
                   xmax=xmax2;
                   xmin=xmin2;
                   ymax=ymax2;
                   ymin=ymin2;
                   ticks=ticks2;
                   s1=s1_2;
                   s2=s2_2;
                   xtick=xtick2;
                   ytick=ytick2;
                   strcpy(xlabel,xlabel2);
                   strcpy(ylabel,ylabel2);
              }
         }
         else
              plot_gf = 0;
      }
}

get_ras(int x,int y)
{
FILE *rasfileptr;
int isize;

        if(strlen(cp_filename) == 0) return;
        strcpy(temp,cp_filename);

again:
        if((rasfileptr=fopen(temp,"r"))==NULL)
        {
                if(strchr(temp,'.') == NULL)
                {
                        strcat(temp,".cp");
                        goto again;
                }

                sprintf(temp,"error opening %s ",cp_filename);
                info_window(x,y,temp);
                plot_ras=0;
                return;
        }
        else
        {
             /* need to open proper file */
             if(ras_type == 0) /* auto */
             {
                filetype=1;
                if(strstr(cp_filename,".img") != NULL ) filetype=2;
                if(strstr(cp_filename,".cmp") != NULL ) filetype=3;
#ifdef SGI
                if(strstr(cp_filename,".sgi") != NULL ) filetype=4;
                if(strstr(cp_filename,".rgb") != NULL ) filetype=4;
#endif
             }
             else
             {
                filetype=ras_type;
             }

             if(filetype==1)  /* CP file */
             {
                if( readcphead(rasfileptr) != -1)
                {
                     sprintf(temp,"CP file of %i x %i read",rasinfo.npx,rasinfo.npy);
                     info_window(x,y,temp);
                     plot_ras=1;
                }
                else
                {
                     sprintf(temp,"CP header read failed on %s",cp_filename);
                     info_window(x,y,temp);
                     plot_ras=0;
                }
                fclose(rasfileptr);

             }

             if((filetype==2)||(filetype==3))  /* IMG and CMP files */
             {
                plot_ras=1;
                rasinfo.xmax=640.;
                rasinfo.xmin=0.;
                rasinfo.ymax=480.;
                rasinfo.ymin=0.;
                rasinfo.npx=512;
                rasinfo.npy=480;
                zmax=255;
                zmin=0;
                plot_zscale=0;
                aspect_ratio = 640./480.;

                sprintf(temp,"File %s read",cp_filename);
                info_window(x,y,temp);
                fclose(rasfileptr);
             }

#ifdef SGI
             if(filetype==4)  /* SGI file */
             {
                fclose(rasfileptr);

                sgiimage = iopen(cp_filename,"r");

                if(sgiimage == NULL)
                {
                        info_window(x,y,"Cannot open file as SGI image");
                        plot_ras=0;
                        return;
                }

                rasinfo.npx = sgiimage->xsize;
                rasinfo.npy = sgiimage->ysize;
                sgi_nchan = sgiimage->zsize;

                plot_ras=1;
                rasinfo.xmax=rasinfo.npx;
                rasinfo.xmin=0.;
                rasinfo.ymax=rasinfo.npy;
                rasinfo.ymin=0.;
                zmax=255;
                zmin=0;
                plot_zscale=0;
                aspect_ratio = ((float)rasinfo.npx)/((float)rasinfo.npy); /* square pixels */

                sprintf(temp,"SGI image of %i x %i opened",rasinfo.npx,rasinfo.npy);
                info_window(x,y,temp);

                iclose(sgiimage);

             }
#endif

             if((plot_gf == 0)&&(read_set==0))
             {
                xmax=rasinfo.xmax;
                xmin=rasinfo.xmin;
                ymax=rasinfo.ymax;
                ymin=rasinfo.ymin;
                s1=0;
                s2=0;
                ticks= 'n';
                xtick = fabs(xmax-xmin)/10.;
                ytick = fabs(ymax-ymin)/10.;
                strcpy(filecurrent,cp_filename);
                plot_lists_defaults(PLOTWIN);
             }
        }

}

get_cp2(int x,int y)
{
FILE *rasfileptr;
struct RASINFO rasinfo2;
int match;

        if(strlen(cp2_filename) == 0) return;
        strcpy(temp,cp2_filename);

again:
        if((rasfileptr=fopen(temp,"r"))==NULL)
        {
                if(strchr(temp,'.') == NULL)
                {
                        strcat(temp,".cp");
                        goto again;
                }

                sprintf(temp,"error opening %s ",cp2_filename);
                info_window(x,y,temp);
                plot_cp2=0;
                return;
        }

        rasinfo2=rasinfo;  /* copy the info */

        if( readcphead(rasfileptr) != -1)
        {
             match=1;
             if(rasinfo.xmax != rasinfo2.xmax) match = 0;
             if(rasinfo.xmin != rasinfo2.xmin) match = 0;
             if(rasinfo.ymax != rasinfo2.ymax) match = 0;
             if(rasinfo.ymin != rasinfo2.ymin) match = 0;
             if(rasinfo.npx != rasinfo2.npx) match = 0;
             if(rasinfo.npy != rasinfo2.npy) match = 0;

             if(match == 1)
             {
                  sprintf(temp,"Matching CP file of %i x %i read",rasinfo.npx,rasinfo.npy);
                  info_window(x,y,temp);
                  plot_cp2=1;
             }
             else
             {
                 info_window(x,y,"CP files don't match");
                 plot_cp2=0;
                 rasinfo=rasinfo2; /* put it back */
             }
        }
        else
        {
             sprintf(temp,"CP header read failed on %s",cp2_filename);
             info_window(x,y,temp);
             plot_cp2=0;
             rasinfo=rasinfo2; /* put it back */
        }

        fclose(rasfileptr);

}

read_settings(int x,int y)
{
FILE *setfileptr;
int i,k;

      if(strlen(set_filename)==0) return(-1);

      plot_lists_defaults(PLOTWIN);

      if((setfileptr=fopen(set_filename,"r")) == NULL)
      {
          sprintf(temp,"Read of %s failed",set_filename);
          info_window(x,y,temp);
          read_set=0;
          return;
      }

      /* z scaling info */
      r_set_f(setfileptr,&zmax);
      r_set_f(setfileptr,&zmin);
      r_set_s(setfileptr,zlabel);
      r_set_f(setfileptr,&zstep);
      r_set_i(setfileptr,&s3);
      r_set_f(setfileptr,&znull);
      r_set_f(setfileptr,&zbackground);
      r_set_i(setfileptr,&zback);
      r_set_i(setfileptr,&plot_zscale);
      r_set_i(setfileptr,&zdigits);
      r_set_i(setfileptr,&colormap);

      /* file info */
      r_set_s(setfileptr,gf_filename);
      r_set_s(setfileptr,cp_filename);
      r_set_s(setfileptr,cp2_filename);
      r_set_i(setfileptr,&ras_type);
      r_set_i(setfileptr,&fade);
      r_set_i(setfileptr,&fade_steps);

      /* X-Y scaling */
      r_set_f(setfileptr,&xmax);
      r_set_f(setfileptr,&xmin);
      r_set_f(setfileptr,&ymax);
      r_set_f(setfileptr,&ymin);
      r_set_s(setfileptr,xlabel);
      r_set_s(setfileptr,ylabel);
      r_set_c(setfileptr,&ticks);
      r_set_f(setfileptr,&xtick);
      r_set_f(setfileptr,&ytick);
      r_set_i(setfileptr,&s1);
      r_set_i(setfileptr,&s2);
      r_set_c(setfileptr,&xzero);
      r_set_c(setfileptr,&yzero);

      /* GF file i/o settings */
      r_set_i(setfileptr,&use_eof);
      r_set_i(setfileptr,&use_delim);
      r_set_c(setfileptr,&delim_char);
      r_set_i(setfileptr,&dlen);
      r_set_i(setfileptr,&vlen);

      /* plotting options */
      r_set_f(setfileptr,&arrow_size);
      r_set_f(setfileptr,&symbol_scale);
      r_set_i(setfileptr,&psjoin);
      r_set_f(setfileptr,&vp);
      r_set_i(setfileptr,&encap);
      r_set_c(setfileptr,&rotp);
      r_set_i(setfileptr,&axes_color);
      r_set_i(setfileptr,&axes_width);

      r_set_i(setfileptr,&plot_axes);
      r_set_i(setfileptr,&plot_comments);
      r_set_i(setfileptr,&plot_curves);

      r_set_i(setfileptr,&k);
      for(i=0;i<k;i++)
      {
          r_set_i(setfileptr,&seg_color[i]);
          r_set_i(setfileptr,&seg_width[i]);
          r_set_i(setfileptr,&seg_pattern[i]);
          r_set_i(setfileptr,&seg_pat_len[i]);
       }

       r_set_i(setfileptr,&num_comments);
       for(i=0;i<(num_comments+1);i++)
       {
          r_set_i(setfileptr,&text_font[i]);
          r_set_i(setfileptr,&text_color[i]);
       }

       r_set_f(setfileptr,&x_plot_size);
       r_set_f(setfileptr,&y_plot_size);

       fclose(setfileptr);

       info_window(x,y,"Settings read in");

       read_set=1;
}

write_settings(int x,int y)
{
FILE *setfileptr;
int i,k;

      if(strlen(set_filename)==0) return(-1);

      if((setfileptr=fopen(set_filename,"w")) == NULL)
      {
          info_window(x,y,"Write failed");
          return;
      }

      /* z scaling info */
      w_set_f(setfileptr,zmax);
      w_set_f(setfileptr,zmin);
      w_set_s(setfileptr,zlabel);
      w_set_f(setfileptr,zstep);
      w_set_i(setfileptr,s3);
      w_set_f(setfileptr,znull);
      w_set_f(setfileptr,zbackground);
      w_set_i(setfileptr,zback);
      w_set_i(setfileptr,plot_zscale);
      w_set_i(setfileptr,zdigits);
      w_set_i(setfileptr,colormap);

      /* file info */
      w_set_s(setfileptr,gf_filename);
      w_set_s(setfileptr,cp_filename);
      w_set_s(setfileptr,cp2_filename);
      w_set_i(setfileptr,ras_type);
      w_set_i(setfileptr,fade);
      w_set_i(setfileptr,fade_steps);

      /* X-Y scaling */
      w_set_f(setfileptr,xmax);
      w_set_f(setfileptr,xmin);
      w_set_f(setfileptr,ymax);
      w_set_f(setfileptr,ymin);
      w_set_s(setfileptr,xlabel);
      w_set_s(setfileptr,ylabel);
      w_set_c(setfileptr,ticks);
      w_set_f(setfileptr,xtick);
      w_set_f(setfileptr,ytick);
      w_set_i(setfileptr,s1);
      w_set_i(setfileptr,s2);
      w_set_c(setfileptr,xzero);
      w_set_c(setfileptr,yzero);

      /* GF file i/o settings */
      w_set_i(setfileptr,use_eof);
      w_set_i(setfileptr,use_delim);
      w_set_c(setfileptr,delim_char);
      w_set_i(setfileptr,dlen);
      w_set_i(setfileptr,vlen);

      /* plotting options */
      w_set_f(setfileptr,arrow_size);
      w_set_f(setfileptr,symbol_scale);
      w_set_i(setfileptr,psjoin);
      w_set_f(setfileptr,vp);
      w_set_i(setfileptr,encap);
      w_set_c(setfileptr,rotp);
      w_set_i(setfileptr,axes_color);
      w_set_i(setfileptr,axes_width);

      w_set_i(setfileptr,plot_axes);
      w_set_i(setfileptr,plot_comments);
      w_set_i(setfileptr,plot_curves);

      k=num_data_segments();
      w_set_i(setfileptr,k);
      for(i=0;i<k;i++)
      {
          w_set_i(setfileptr,seg_color[i]);
          w_set_i(setfileptr,seg_width[i]);
          w_set_i(setfileptr,seg_pattern[i]);
          w_set_i(setfileptr,seg_pat_len[i]);
       }

       w_set_i(setfileptr,num_comments);
       for(i=0;i<(num_comments+1);i++)
       {
          w_set_i(setfileptr,text_font[i]);
          w_set_i(setfileptr,text_color[i]);
       }

       w_set_f(setfileptr,x_plot_size);
       w_set_f(setfileptr,y_plot_size);

       fclose(setfileptr);

       info_window(x,y,"Settings written");
}

w_set_i(fileptr,n)
int n;
FILE *fileptr;
{
        fprintf(fileptr,"%i\n",n);
}


w_set_f(fileptr,f)
float f;
FILE *fileptr;
{
        fprintf(fileptr,"%g\n",f);
}

w_set_s(fileptr,s)
char *s;
FILE *fileptr;
{
        fprintf(fileptr,"%s\n",s);
}

w_set_c(fileptr,c)
char c;
FILE *fileptr;
{
        fprintf(fileptr,"%c\n",c);
}

r_set_i(fileptr,n)
int *n;
FILE *fileptr;
{
char line[100];
int len;
        len = 100;
        fgets(line,len,fileptr);
        *n=atoi(line);
}

r_set_f(fileptr,f)
float *f;
FILE *fileptr;
{
char line[100];
int len;
        len = 100;
        fgets(line,len,fileptr);
        *f=atof(line);
}

r_set_s(fileptr,s)
char *s;
FILE *fileptr;
{
char line[100];
int len;
        len = 100;
        fgets(line,len,fileptr);
        strip_crlf(line); /* strip any \r or \n from string */
        strcpy(s,line);
}


r_set_c(fileptr,c)
char *c;
FILE *fileptr;
{
char line[100];
int len;
        len = 100;
        fgets(line,len,fileptr);
        *c=line[0];
}


usage(cmd)
     char       *cmd;
{
  fprintf(stderr, "usage: %s [-v] [-h] [-g geometry] [-d display] [filename]\n", cmd);
}

defaults()
{

read_set=0;/* Haven't yet read in settings */

fade = 0; /* no fade */
fade_steps=10;
ras_type=0; /* auto detect */

npmax=MAXPOINTS-1;      /* one less than the number in the array dimension */
dlen=10;          /* the data length and invec length formats */
vlen=3;
xcol=1;
ycol=2;
ticks='n';                 /* show grids (g) or ticks (t) or none (n) */
s1=0;                  /* linear = 0 or log = 1 x scaling */
s2=0;                  /*                       y         */
fheight=13;       /* should get these from x font */
fwidth=8;         /* set for 8x13 font set below */
num_comments=0;
xzero='y';   /* show the x = 0 line ? */
yzero='y';
vp=1;      /* the variable plot size for text */
device=3;   /* 1=screen, 3=post, 2=hpgl output */
encap = 0;  /* default to non encapsulated post */
rotp='n';   /* default to nonrotated plot output */
cmdline=0;         /* is there a filename on the cmd line? 1=y */
currfont=2;
patlen=4.;         /* pattern length for line types */
linetype=1;         /* linetype or screen color */
use_delim=0;     /* use delimeter charater on reads 0=no */
delim_char=',';    /* delimeter character */
arrow_size = 0.7;   /* size of invec 11&12 arrows, 0 = scale with vector */
symbol_scale = 1.0 ; /* scale factor for symbols */
psjoin = 1; /* 0 = miter 1=round 2=bevel joins in ps output */
axes_color=0; /* black */
axes_width=1; /* 12 */

s3=0; /* linear z axis default */
znull = 999; /* null z value */
zbackground = 0; /* data for background color */
zback = 0;/* use white  for ras background */
plot_zscale=1; /* yes */
zdigits=5; /* number of format digits */

strcpy(hex_nibble,"0123456789ABCDEF");

strcpy(gf_filename,"");
strcpy(cp_filename,"");
strcpy(cp2_filename,"");
strcpy(set_filename,"");

plot_axes=1; /* 1 = yes 0 = no */
plot_comments=1; /* 1 = yes 0 = no */
plot_curves=1; /* 1 = yes 0 = no */
plot_device=3; /* ps default */
plot_ras = 0;
plot_gf = 0;
plot_cp2=0;

x_plot_size=5.;      /* plot  sizes */
y_plot_size=5.;      /* plot  sizes */
aspect_ratio=1.;

/* set the input window structure */
set_in_wins();

set_hex(); /* set up the hex conversion strings */

}

set_hex()
{
int i,j;

        for(i=0;i<16;i++)
        {
                for(j=0;j<16;j++)
                {
                        hex_byte[i*16+j][0]=hex_nibble[i];
                        hex_byte[i*16+j][1]=hex_nibble[j];
                        hex_byte[i*16+j][2]='\0';
                 }
         }
}

alltops(FILE *file,FILE *file2,int x,int y)
/* 
	file2 = file to read for ras data 
*/
{
int xtrans,ytrans,t,iz;
float sizex,sizey;
/*
        SET UP  AND SEND OUT IMAGES
*/

     setcolormap();

     maptype = 0; /* color 24 bit */
     if((colormap == 0) || (filetype == 2)) maptype =1; /* grayscale 8 bit gray */
     if(filetype == 3) maptype=2;  /* cmp file 1 bit b/w*/

     if((filetype == 2 )||(filetype == 3) )  flip = 1; /*  'img and cmp need flipping */

#ifdef SGI
     if(filetype == 4)  /* for sgi files */
     {
        if(sgi_nchan < 3)
                maptype = 1; /* gray scale */
        else
                maptype = 0; /* color */
     }
#endif

     if(zback == 1)
     {

     /* should use zhex ? */
        if(maptype == 0)
        {
                iz = zscale(zbackground);
                rgb(white,iz);
        }
        else
                strcpy(white,hex_byte[zscale(zbackground)]);
      }
      else
      {
             if( maptype == 0 ) strcpy(white,"ffffff"); /*  color 3x8 bit */
             if( maptype == 1 ) strcpy(white,"ff"); /*  grayscale 8 bit */
      }

     strcpy(pout," gsave ");
     plot_out(pout);

         ps_setimageclip(); /*  save clip path, set new clip path */
/*
         sizex = x_plot_size * fabs((rasinfo.xmax - rasinfo.xmin) / (xmax - xmin));
         sizey = x_plot_size * aspect_ratio * fabs((rasinfo.ymax - rasinfo.ymin)/(ymax-ymin));
*/
         sizex = (xr-xl) * fabs((rasinfo.xmax - rasinfo.xmin)/(xmax - xmin));
         sizey = (yu-yb) * fabs((rasinfo.ymax - rasinfo.ymin)/(ymax-ymin));

         xtrans=x_coord(rasinfo.xmin);

         if(flip == 0 )
         {
              ytrans=y_coord(rasinfo.ymin);
         }
         else
         {
              ytrans=y_coord(rasinfo.ymax);
         }

         ps_startimage(xtrans,ytrans,maptype,rasinfo.npx,rasinfo.npy,&sizex,&sizey);

         flip = 0;/*    don't flip in the zaxes */

         if((filetype == 1)&&(plot_cp2==0)) imagecp(file);
         if((filetype == 1)&&(plot_cp2==1)) image2cp(file,file2);
         if(filetype == 2) imageimg(file,x,y);
         if(filetype == 3) imagecmp(file,x,y);
#ifdef SGI
         if(filetype == 4) imagesgi();
#endif

         ps_endimage(xtrans,ytrans,&sizex,&sizey);

         /*   CLOSE #3   do this outside this sub */

         strcpy(pout," grestore ");
         plot_out(pout); /*  to restore clip path */

         zaxis();

}

setcolormap()
{
int i,map,ntsc,r2,b2,g2;

    map = colormap;
    ntsc = 0; /* do we adjust colors to be NTSC legal ? */

    if(colormap < 1 ) return;
    if(colormap > 5)
    {
        map=colormap-5;
        ntsc = 1;
    }

    for(i=0;i<256;i++)
    {

         if(map == 1)
         {

                if(i <= 85 )
                {
                        red[i] = 0;
                        green[i] = 0;
                        blue[i] = i;
                }

                if( i > 85 && i <= 170 )
                {
                        red[i] = 0;
                        green[i] = i;
                        blue[i] = 0;
                }

                if( i > 170 )
                {
                        red[i] = i;
                        green[i] = 0;
                        blue[i] = 0;
                }
         }

        if( map == 2 )
        {
               if( i <= 85 )
               {
                       red[i] = 0;
                       green[i] = 0;
                       blue[i] = i * 3;
               }

               if( i > 85 && i <= 170 )
               {
                       red[i] = 0;
                       green[i] = (i - 86) * 3;
                       blue[i] = 255;
               }

               if( i > 170 )
               {
                       red[i] = (i - 171) * 3;
                       green[i] = 255;
                       blue[i] = 255;
               }
        }

        if( map == 3 )
        {

               if( i <= 63 )
               {
                       red[i] = 0;
                       green[i] = i * 4;
                       blue[i] = 255;
               }

               if( i > 63 && i <= 127 )
               {
                       red[i] = 0;
                       green[i] = 255;
                       blue[i] = 255 - (i - 64) * 4;
               }

               if( i > 127 && i <= 191 )
               {
                       red[i] = (i - 128) * 4;
                       green[i] = 255;
                       blue[i] = 0;
               }

               if( i > 191 && i <= 255 )
               {
                       red[i] = 255;
                       green[i] = 255 - (i - 192) * 4;
                       blue[i] = 0;
               }

        }

        if( map == 4 )
        {

               if( i <= 51 )
               {
                       red[i] = 255 - i * 5;
                       green[i] = 0;
                       blue[i] = 255;
               }

               if( i > 51 && i <= 102 )
               {
                       red[i] = 0;
                       green[i] = (i - 51) * 5;
                       blue[i] = 255;
               }

               if( i > 102 && i <= 153 )
               {
                       red[i] = 0;
                       green[i] = 255;
                       blue[i] = 255 - (i - 102) * 5;
               }

               if( i > 153 && i <= 204 )
               {
                       red[i] = (i - 153) * 5;
                       green[i]= 255;
                       blue[i] = 0;
               }

               if( i > 204 && i <= 255 )
               {
                       red[i] = 255;
                       green[i] = 255 - (i - 204) * 5;
                       blue[i] = 0;
               }

        }

        if( map == 5 )
        {

               if( i <= 41 )
               {
                       red[i] = 255;
                       green[i] = 0;
                       blue[i] = 255;
               }

               if( i > 41 && i <= 85 )
               {
                       red[i] = 0;
                       green[i] = 0;
                       blue[i] = 255;
               }

               if( i > 85 && i <= 127 )
               {
                       red[i] = 0;
                       green[i] = 255;
                       blue[i] = 255;
               }

               if( i > 127 && i <= 170 )
               {
                       red[i] = 0;
                       green[i] = 255;
                       blue[i] = 0;
               }

               if( i > 170 && i <= 212 )
               {
                       red[i] = 255;
                       green[i] = 255;
                       blue[i] = 0;
               }

               if( i > 212 && i <= 255 )
               {
                       red[i] = 255;
                       green[i] = 0;
                       blue[i] = 0;
               }

        }

        if(ntsc == 1) /* make the colors legal */
        {
             make_legal_yiq_i(red[i],green[i],blue[i],&r2,&g2,&b2);
             red[i]=r2;
             green[i]=g2;
             blue[i]=b2;
        }


     }  /* i loop */

}


imagecmp(FILE *filecmp,int x,int y)
{
int i,j,fchk;
unsigned char ai[520];
/*
   READ, CONVERT TO HEX AND WRITE
*/

       for (i = 1;i<=480;i++)
       {

                 fchk=fread(&ai[0],sizeof(ai[0]),64,filecmp);
                 if(fchk != 64)
                 {
                        info_window(x,y,"*** error on CMP file read ***");
                        /* do close in calling sub */
                        return(-1);
                 }
                 strcpy(pout," ");
                 for(j=0;j<64;j++)
                 {
                     strcat(pout,hex_byte[ai[j]]);
                 }
                 plot_out(pout);

       }

       return(0);
}

imagecp(filecp)
FILE *filecp;
{
int i,j,len,l,ncpcol;
char ai[100];
char *tptr;
float z;

/*
   READ CP file, CONVERT TO HEX AND WRITE
*/
     len=100;

     /* ' read first entry for auto # cols detect */

     fgets(ai,len,filecp);

     if( strlen(ai) > 11 )
     {
        ncpcol = 13 ;
     }
     else
     {
         ncpcol = 0;
     }

     for(l = 1;l<=rasinfo.npy;l++)
     {
         strcpy(pout," ");

         for( i = 1;i<=rasinfo.npx;i++)
         {

             if( i != 1 || l != 1 ) fgets(ai,len,filecp);

             if(strlen(ai) == 0 ) return(-1);

             tptr=&ai[ncpcol];
             tptr[10]='\0';
             z = atof(tptr);

             zhex(temp,z);
             strcat(pout,temp);

             if( strlen(pout) > 80 )
             {
                    plot_out(pout);
                    strcpy(pout, " ");
             }
         }
         plot_out(pout);
      }
}

image2cp(filecp,filecp2)
FILE *filecp,*filecp2;
{
int i,j,len,l,ncpcol,ncpcol2;
char ai[100],ai2[100];
char *tptr;
float z,z2,f;

     len=100;

/*
   READ CP file, CONVERT TO HEX AND WRITE
*/

     /* ' read first entry for auto # cols detect */

     fgets(ai,len,filecp);

     if( strlen(ai) > 11 )
     {
        ncpcol = 13 ;
     }
     else
     {
         ncpcol = 0;
     }

     /* ' read first entry for auto # cols detect */

     fgets(ai2,len,filecp2);

     if( strlen(ai2) > 11 )
     {
        ncpcol2 = 13 ;
     }
     else
     {
         ncpcol2 = 0;
     }

     for(l = 1;l<=rasinfo.npy;l++)
     {
         strcpy(pout," ");

         for( i = 1;i<=rasinfo.npx;i++)
         {

             if( (i != 1) || (l != 1) ) fgets(ai,len,filecp);
             if( (i != 1) || (l != 1) ) fgets(ai2,len,filecp2);

             if(strlen(ai) == 0 ) return(-1);
             if(strlen(ai2) == 0 ) return(-1);

             tptr=&ai[ncpcol];
             tptr[10]='\0';
             z = atof(tptr);

             tptr=&ai2[ncpcol2];
             tptr[10]='\0';
             z2 = atof(tptr);

             f=(float)(fade_index-1)/(float)(fade_steps-1);

             if( (z == znull)||(z2 == znull))
             {
                   if(f == 1.0) z=z2;
             }
             else
             {
                   z=(1.-f)*z+f*z2;
             }

             zhex(temp,z);
             strcat(pout,temp);

             if( strlen(pout) > 80 )
             {
                    plot_out(pout);
                    strcpy(pout, " ");
             }
         }
         plot_out(pout);
      }
}


imageimg(FILE *fileimg,int x,int y)
{
int i,j,k,fchk;
unsigned char ai[520];

/*
 REM   READ, CONVERT TO HEX AND WRITE
*/

       for (i = 1;i<=480;i++)
       {

           for(k=0;k<8;k++)  /* 8*64 = 512 */
           {
                 fchk=fread(&ai[0],sizeof(ai[0]),64,fileimg);
                 if(fchk != 64)
                 {
                        info_window(x,y,"*** error on IMG file read ***");
                        /* do close in calling sub */
                        return(-1);
                 }
                 strcpy(pout," ");
                 for(j=0;j<64;j++)
                 {
                     strcat(pout,hex_byte[ai[j]]);
                 }
                 plot_out(pout);
           }

       }

       return(0);
}

#ifdef SGI

imagesgi()
{
/*
        sgiimage is global

*/
int i,j,k,fchk;
short rbuf[800],gbuf[800],bbuf[800],obuf[800];

/*
 REM   READ, CONVERT TO HEX AND WRITE
*/

        for(i=0; i<rasinfo.npy; i++)
        {
               if(sgi_nchan < 3)   /* gray scale */
               {
                       getrow(sgiimage,obuf,i,0);
               }
               else
               {
                       getrow(sgiimage,rbuf,i,0);
                       getrow(sgiimage,gbuf,i,1);
                       getrow(sgiimage,bbuf,i,2);
               }

               strcpy(pout," ");

               for(j=0;j<rasinfo.npx;j++)
               {
                   if(sgi_nchan < 3)      /* grayscale */
                   {
                        strcat(pout,hex_byte[obuf[j]]);
                   }
                   else
                   {
                        strcat(pout,hex_byte[rbuf[j]]);
                        strcat(pout,hex_byte[gbuf[j]]);
                        strcat(pout,hex_byte[bbuf[j]]);
                   }

                   if(strlen(pout) > 80 )
                   {
                        plot_out(pout);
                        strcpy(pout," ");
                   }
               }
               plot_out(pout);
       }

       return(0);
}
#endif

zaxis()
{
int t,tx,ty,nz,k,npx,npy;
float z3,z4,z,sizex,sizey;
char fmt[10];
/*
    PRINT Z SCALE INFO
*/


if( zstep <= 0 ) return;
if( plot_zscale == 0 ) return;

if( filetype != 1 ) return; /* cmp file only has b/w */

 t = 0;
 /*
 tx = xr +(int)(.1 * (float)(xr - xl));
 */
 tx = xr + 2*ps_fheight;
 ty = yb;

 nz = (int)(fabs(zmax - zmin) / zstep + .5) + 1;

 if(zmax > zmin)
        z3 = 1;
 else
        z3=-1;

 /*need to match to outside box not the image */
 sizey = yu-yb;
 sizex = sizey / (2 * nz - 1);

 npx = 1;
 npy = 2 * nz - 1;

if( maptype == 0 ) strcpy(white,"ffffff"); /*  color 3x8 bit */
if( maptype == 1 ) strcpy(white,"ff"); /*  grayscale 8 bit */

ps_startimage(tx, ty, maptype,npx,npy,&sizex,&sizey);

strcpy(pout," ");

 if( zmin < zmax )
    z4 = zmin;
 else
    z4 = zmax;


 for(k=nz;k>=1;k--)
 {
        z = (float)(nz - k) * zstep + z4;
        zhex(temp,z);
        strcat(pout,temp);
        strcat(pout," ");
        if(k > 1)
        {
            strcat(pout,white);
            strcat(pout," ");
        }
 }

 plot_out(pout);
 ps_endimage(tx, ty,&sizex,&sizey);

 zbox(tx, ty,nz,sizex,sizey);

 set_font(0); /* text = 0 is axes labels */

 tx = tx + (3 * sizex)/2;

 for(k=1;k<=nz;k++)
 {
        z = (float)(k - 1) * zstep + z4;

        if( fabs(z / zstep) > .01 )
             convert(temp,z,zdigits);
        else
             strcpy(temp,"0.0");

        t = 0;
        ty = (2 * k - 2) * sizey / (2 * nz - 1) + yb;
        ps_text(tx, ty, temp, t);

 }

/*
 tx = tx - (3 * sizex)/2 - 100 * vp;
 */
 tx = tx - (3*sizex)/2 - ps_fheight/2;
 ty = (yu + yb) / 2 - 52 * vp * get_strlen(zlabel);
 t = 90;
 ps_text(tx, ty, zlabel, t);

 setcolor(0);  /* black is the default */

}


zbox (tx, ty,nz,sizex,sizey)
int tx,ty,nz;
float sizex,sizey;
{
           int t,k;

           t = 1; /* 28 */
           if((axes_width == 2)||(axes_width == 0)) t =6; /*36 */
           ps_setpen(t);

           for(k=1;k<=nz;k++)
           {
                   t = 5;
                   xy[1] = tx;
                   xy[2] = (2 * k - 2) * sizey / (2 * nz - 1) + ty;
                   xy[3] = xy[1] + sizex;
                   xy[4] = xy[2];
                   xy[5] = xy[3];
                   xy[6] = xy[4] + sizey / (2 * nz - 1);
                   xy[7] = xy[5] - sizex;
                   xy[8] = xy[6];
                   xy[9] = xy[7];
                   xy[10] = xy[8] - sizey / (2 * nz - 1);

                   ps_line(t);
           }

           t=2;
           ps_setpen(t); /* default of 12*/
}

zhex(out,z)
float z;
char *out;
{
         int iz;

         if( z == znull )
         {
             strcpy(out,white);  /* set blank out to white */
             return;
         }

         iz = zscale(z);

         if( maptype == 0 )
            rgb(out,iz);  /*     ' color 3x8 rgb */
         else
            strcpy(out,hex_byte[iz]);/* ' greyscale 8 bits also works for cmp 1 bit packed */

}

int zscale (z)
float z;
{
         int iz;

         if( s3 == 0 )    /* log Z scaling switch */
               iz = (z - zmin) / (zmax - zmin) * 255;
         else
               iz = (log(z) - log(zmin)) / (log(zmax) - log(zmin)) * 255;

         if( iz > 255 ) iz = 255;

         if( iz < 0 ) iz = 0;

         return(iz);

}


int readcphead(filecp)
FILE *filecp;
{
/*
        reads a cpfile header returns

        0 = succuss
        -1 = failure

        rasinfo.(xmax,xmin,ymax,ymin,npx,npy,np)

*/

         char line[50],temp[50],*tptr;
         int len;

         len = 50;
/*
REM GET FILE HEADER
LINE INPUT #3, ai$: u2 = VAL(MID$(ai$, 16, 10)): u1 = VAL(MID$(ai$, 26, 10))
LINE INPUT #3, ai$: npx = VAL(MID$(ai$, 16, 10))
LINE INPUT #3, ai$: v2 = VAL(MID$(ai$, 16, 10)): v1 = VAL(MID$(ai$, 26, 10))
LINE INPUT #3, ai$: npy = VAL(MID$(ai$, 16, 10))
LINE INPUT #3, ai$: NPCP = VAL(ai$)
*/

         fgets(line,len,filecp);

         strcpy(temp,line);
         tptr=&temp[15];
         tptr[10]='\0';
         rasinfo.xmax=atof(tptr);

         strcpy(temp,line);
         tptr=&temp[25];
         tptr[10]='\0';
         rasinfo.xmin=atof(tptr);

         fgets(line,len,filecp);

         strcpy(temp,line);
         tptr=&temp[15];
         tptr[10]='\0';
         rasinfo.npx=atoi(tptr);

         fgets(line,len,filecp);

         strcpy(temp,line);
         tptr=&temp[15];
         tptr[10]='\0';
         rasinfo.ymax=atof(tptr);

         strcpy(temp,line);
         tptr=&temp[25];
         tptr[10]='\0';
         rasinfo.ymin=atof(tptr);

         fgets(line,len,filecp);

         strcpy(temp,line);
         tptr=&temp[15];
         tptr[10]='\0';
         rasinfo.npy=atoi(tptr);

         fgets(line,len,filecp);

         rasinfo.np=atoi(line);

         if( (rasinfo.xmin == 0) && (rasinfo.xmax == 0) )
         {
           printf( "Bad Cp file \n");
           return(-1);
         }

         return(0);

}


rgb(out,i)
char *out;
int i;
{
         strcpy(out,"");

         strcat(out,hex_byte[red[i]]);
         strcat(out,hex_byte[green[i]]);
         strcat(out,hex_byte[blue[i]]);

}

ps_setimageclip()
{
int t,iz;
/*
REM    BOX AROUND WHOLE IMAGE for clip
*/

         t = 5;

         xy[1] = xl;
         xy[2] = yb;

         xy[3] = xr;
         xy[4] = yb;

         xy[5] = xr;
         xy[6] = yu;

         xy[7] = xl;
         xy[8] = yu;

         xy[9] = xl;
         xy[10] = yb;

         ps_clip(t);

/*  fill the clip path with the background color if( not white */

         if( maptype == 1 )
         {
                 if( zback == 1 )
                 {
                         iz = zscale(zbackground);
                         rgb(pout,iz);
                         strcat(pout," setrgbcolor clippath fill ");
                         plot_out(pout);
                         strcpy(pout," 0 setgray "); /* ' set back to black default color */
                         plot_out(pout);
                 }
         }

         if( maptype == 2 )
         {
                 if( zback == 1 )
                 {
                         iz = zscale(zbackground);
                         sprintf(pout,"%g setgray clippath fill " ,((float)iz + 1.) / 255.);
                         plot_out(pout);
                         strcpy(pout," 0 setgray ");/* ' set back to black default color */
                         plot_out(pout);
                 }
         }


}

ps_clip (n)
int n;
{
int tx,ty,i;
/*
 LINE TO X,Y FROM CURRENT POINT to set up clip path
*/

 tx = xy[1];
 ty = xy[2];
 plot_rotate(&tx,&ty);
 sprintf(pout," n %i %i m ",tx,ty);
 plot_out(pout);

 for(i=2;i<=n;i++)
 {
      tx = xy[2 * i - 1];
      ty = xy[2 * i];
      plot_rotate(&tx,&ty);
      sprintf(pout," %i %i l ",tx,ty);
      plot_out(pout);
  }

  strcpy(pout," clip n ");
  plot_out(pout);

}

ps_endimage (tx, ty,sizex,sizey)
int tx,ty;
float *sizex,*sizey;
{
int ttx,tty;
/*

    RESET AFTER IMAGE

*/
        ttx=tx;
        tty=ty;
        plot_rotate(&ttx,&tty);

        if(rotp == 'n')
        {
             sprintf(pout," %g %g scale %i %i translate ",1/(*sizex),1/(*sizey),-ttx,-tty);
        }
        else
        {
             sprintf(pout," %g %g scale %i %i translate ",1/(*sizey),1/(*sizex),-ttx,-tty);
        }
        plot_out(pout);

}


ps_startimage (tx, ty, t,npx,npy,sizex,sizey)
int tx,ty,t,npx,npy;
float *sizex,*sizey;
{
int ttx,tty;
/*
       INIT IMAGE color or grayscale (8/1 bits)
*/
         ttx=tx;
         tty=ty;
         plot_rotate(&ttx,&tty);

         sprintf(pout,"/picstr %i string def ",npx);
         plot_out(pout);
         sprintf(pout," %i %i translate ",ttx,tty);
         plot_out(pout);
         if(rotp == 'n')
         {
              sprintf(pout," %g %g scale ",*sizex,*sizey);
         }
         else
         {
              sprintf(pout," %g %g scale ",*sizey,*sizex);
         }
         plot_out(pout);

      if(rotp == 'n')
      {

         if( flip == 0 )
         {
              /* color 3x8 bit image */
              if( t == 0 )
                  sprintf(pout," %i %i 8 [%i 0 0 %i 0 0] {currentfile picstr readhexstring pop} false 3 colorimage ",npx,npy,npx,npy);

             /* greyscale 8 bit image */

              if( t == 1 )
                  sprintf(pout," %i %i 8 [%i 0 0 %i 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,npx,npy);

             /* 1 bit b/w image */
              if( t == 2 )
                  sprintf(pout," %i %i 1 [%i 0 0 %i 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,npx,npy);

              plot_out(pout);
         }
         else
         {
             /* flipped for *.img *.cmp files */

              /* color 3x8 bit image */
              if( t == 0 )
                  sprintf(pout," %i %i 8 [%i 0 0 %i 0 0] {currentfile picstr readhexstring pop} false 3 colorimage ",npx,npy,npx,-npy);

             /* greyscale 8 bit image */

              if( t == 1 )
                  sprintf(pout," %i %i 8 [%i 0 0 %i 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,npx,-npy);

             /* 1 bit b/w image */
              if( t == 2 )
                  sprintf(pout," %i %i 1 [%i 0 0 %i 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,npx,-npy);

              plot_out(pout);

         }

      }
      else  /* for rotated plots */
      {

         if( flip == 0 )
         {
              /* color 3x8 bit image */
              if( t == 0 )
                  sprintf(pout," %i %i 8 [0 %i %i 0 0 0] {currentfile picstr readhexstring pop} false 3 colorimage ",npx,npy,-npy,npx);

             /* greyscale 8 bit image */

              if( t == 1 )
                  sprintf(pout," %i %i 8 [0 %i %i 0 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,-npy,npx);

             /* 1 bit b/w image */
              if( t == 2 )
                  sprintf(pout," %i %i 1 [0 %i %i 0 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,-npy,npx);

              plot_out(pout);
         }
         else
         {
             /* flipped for *.img *.cmp files */

              /* color 3x8 bit image */
              if( t == 0 )
                  sprintf(pout," %i %i 8 [0 %i %i 0 0 0] {currentfile picstr readhexstring pop} false 3 colorimage ",npx,npy,npy,npx);

             /* greyscale 8 bit image */

              if( t == 1 )
                  sprintf(pout," %i %i 8 [0 %i %i 0 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,npy,npx);

             /* 1 bit b/w image */
              if( t == 2 )
                  sprintf(pout," %i %i 1 [0 %i %i 0 0 0] {currentfile picstr readhexstring pop} image ",npx,npy,npy,npx);

              plot_out(pout);

         }

      }
}


plot_close_device ()
{
/*
13597 REM
13598 REM  CLOSE COMM PORT
13599 REM
*/
        fclose(fileprt);
}

plot_open_device (char *b)
{
/*
only opens a file for now


13497 REM
13498 REM  OPEN SERIAL COMM PORT, SET PLOTTER LIMITS, NUMBER OF PENS
13499 REM
*/
/*    moved earlier ... userposition etc may change these befor this sub is called
xl = 1607; xr = 10300; yu = 7650; yb = 1467;
*/

        fileprt=fopen(b,"wt");
        if(fileprt == NULL)
        {
                printf("Error on plot output file open \n");
                return(-1);
        }
        else return(0);

}

plot_out (char *p)
{
/*
'
  '   output to file/printer
'
*/
        fprintf(fileprt,"%s \n",p);
}

plot_rotate (int *tx,int *ty)
{
/*
13697 REM
13698 REM    APPLY ROTATION MAP IF rotp = 'Y'
13699 REM
*/
int tt;
        if( rotp == 'y' || rotp == 'Y')
        {
              tt = 10300 - *ty; *ty = *tx;  *tx = tt;
        }
}

num_data_segments()
{
int i,k;

      if(np == 0) return(0);

      k = 1;
      for( i = 2;i<=np;i++){if( iv[i] <= 0) k = k + 1;}
      return(k);
}

int sign(float x)
{
        if(x >= 0.0 )
        {
             return(1);
        }
        else return(-1);
}

void convert(char *data,float x,int digits)
{
char fmt[10];
/*
   used to put axis limit values into string data form

*/
                sprintf(fmt,"%%-.%ig",digits);
                sprintf(data,fmt,x); /* was %.4g format */
}

char *lowercase(char *temp)
{
int l,i;
        l=strlen(temp);
        for(i=0;i<l;i++)
        {
                temp[i]=tolower(temp[i]);
        }
        return(temp);
}

char *uppercase(char *temp)
{
int l,i;
        l=strlen(temp);
        for(i=0;i<l;i++)
        {
                temp[i]=toupper(temp[i]);
        }
        return(temp);
}

/*
**********************************************************************

                 device independant subroutines follow

                 these are just hooks to the proper device
                 dependant subroutines

          !!!!!   HPGL removed !!!!!

**********************************************************************
*/

initgraph ()
{
/*
'12005 ON dv GOTO 12100, 13000, 14000' INIT GRAPHICS
*/

if(device ==  3 ) ps_initg();

}



dline (int t)
{
/*
  ' DRAW LINE 12000
*/

if( device ==  3 )  ps_line(t);

}

text (int tx,int ty,char *tt,int t)
{
/*
'12020 ON dv GOTO 12300, 13100, 14100' TEXT
*/
if(device ==  3 ) ps_text(tx, ty, tt, t);
}

setpen (int t)
{
/*
'12040 ON dv GOTO 12080, 13020, 14020' SELECT PEN
*/
if(device ==  3) ps_setpen(t);

}

penup ()
{
/*
'12050 ON dv GOTO 12080, 13060, 12080' PEN UP
*/

}

pendown ()
{
/*
'12060 ON dv GOTO 12080, 13070, 12080' PEN DOWN
*/
}


speed (int t)
{
/*
  ' PEN SPEED

*/


}

s_set_color(int i)
{
/*

             GET NEW LINE TYPE/COLOR FOR SCREEN ONLY at 1800

             still device independant

*/
int t;

     if( device != 1) return(0);

     linetype = linetype + 1;

     if( i == 1) linetype = 1;

     if( iv[i] < 0) linetype = abs(iv[i]);

     t = linetype; ulinetype(&t); linetype=t;
}

ulinetype (int *t)
{
int t1;
t1=*t;
/*
'12030 ON dv GOTO 12600, 13080, 14080' LINE TYPE
*/

if(device ==  3) ps_linetype(&t1);

*t=t1;
}

setcolor(int t)
{
if(device == 3) ps_color(t);
}

int get_strlen(char *text)
{
if(device == 3 ) return(ps_strlen(text));
}

int psbb(float *tllx,float *tlly,float *turx,float *tury)
{
/*
	ps bounding box for psall
*/	
     if(rotp == 'n' || rotp == 'N')   /* ' figure out bounding box dimensions  */
     {
             *tllx = .5;
             *turx = ((float)((int)yu - (int)yb) / 1016.) + 2.5;
             *tlly = 10.5 - ((float)((int)xr - (int)xl) / 1016.) - 3.5;
             *tury = 10.;
     }
     else
     {
             *tllx = 1.0; /* was 0.5 */
             *tlly = 1.0; /* was 0.5 */
             *turx = ((float)((int)xr - (int)xl) / 1016.) + 3.5; /* was + 2.5 */
             *tury = ((float)((int)yu - (int)yb) / 1016.) + 2.;  /* was + 2.5 */
     }
}
