/*

	menusub by Tom Shield
	
	pop-up menu subroutine
	
	fill then menu_info structure, see menusub.h before calling
	
	call ms_init first to pass display etc into subs.
	
	based on winman example menu code in 
	"Xlib Programming Manual, Vol 1" published by O'Reilly and Associates

 */

#include <stdio.h>
#include <strings.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>

#include "menusub.h"

#define BLACK  1
#define WHITE  0

int debug;

ms_init(Display *display,int screen,char *fore_color,char *back_color,char *font_name)
{
/*
	call to set up X variables for menusub routines
	
	call with ms_init(display,screen,NULL,NULL,NULL) for defaults 
		(black on white with 9x15 font)
	
*/	
XColor color_exact,color_screen;

	if(display == NULL)
	{
		printf("menusub: null display\n"); 
		exit(-1);
	}
	ms_display = display;
	ms_screen = screen;

        ms_colormap = DefaultColormap(ms_display,screen);

       /* default pixel values */
        ms_foreground = BlackPixel(ms_display,screen);
        ms_background = WhitePixel(ms_display,screen);
        
        if(back_color != NULL)
        {
        	if(debug)
        		printf("back pixel name = %s \n",back_color);
        		
        	if(XLookupColor(ms_display,ms_colormap,back_color,&color_exact,&color_screen) != 0)
        	{
        		XAllocColor(ms_display,ms_colormap,&color_screen);
        		ms_background = color_screen.pixel;
        		if(debug)
        			printf(" XlookupColor ok \n");
        	}
        }
        		
        
        if(debug)
        	printf("Background pixel = %i \n",ms_background);
        	
        if(fore_color != NULL)
        {
        	if(debug)
        		printf("foreground pixel name = %s \n",fore_color);
        		
        	if(XLookupColor(ms_display,ms_colormap,fore_color,&color_exact,&color_screen) != 0)
        	{
        		XAllocColor(ms_display,ms_colormap,&color_screen);
        		ms_foreground = color_screen.pixel;
        		if(debug)
        			printf(" XlookupColor ok \n");
        	}
        }
	
	if(font_name != NULL)
	{      
        	ms_font_info = XLoadQueryFont(ms_display,font_name);
        }
        else
        {
        	ms_font_info = XLoadQueryFont(ms_display,MENU_FONT);
        }
        
        if (ms_font_info == NULL)
        {
        	printf("menusub: Cannot open font %s\n",MENU_FONT);
        	exit( -1 );
        }
        
}

int menu_window(int x, int y, int menu )
/*
	call with
		x = x position for menu window
		y = y position for menu window
		menu = index to menu_info structure (0 to MAX_MENUS - 1)
		
	returns
		winindex = 0 to num_items-1 of menu item selected
		winindex = -1 no item selected
		
*/		
		
{
    Window menuwin;
    Window panes[MAX_CHOICE];
    int menu_width, menu_height, border_width = 2;
    int winindex,i;
    int cursor_shape;
    Cursor cursor, hand_cursor;

    int direction, ascent, descent;
    int char_count;
    char *string;
    XCharStruct overall;
    Bool owner_events;
    int pointer_mode;
    int keyboard_mode;
    Window confine_to;
    GC gc, rgc,ggc;
    int pane_height;
    Window assoc_win,inverted_pane;
    XEvent event;
    unsigned int button;
    
    unsigned long valuemask;
    XSetWindowAttributes attributes;
    XColor color_exact,color_screen;


	/* find longest menu label to set menu width */
    	char_count = 0;
    	for(i=0;i < menu_info[menu].num_items;i++)
    	{
    		if(strlen(menu_info[menu].label[i]) > char_count) 
    		{
    			string = menu_info[menu].label[i];
    			char_count = strlen(menu_info[menu].label[i]);
    		}
    	}
    
    if(debug)
    	printf(" char_count = %i \n",char_count);

    /* determine the extent of each menu pane based on the font size */
    XTextExtents(ms_font_info, string, char_count, &direction, &ascent, 
            &descent, &overall);

    menu_width = overall.width + H_BORDER*2;
    pane_height = ms_font_info->max_bounds.ascent + ms_font_info->max_bounds.descent + V_BORDER*2;
    menu_height = pane_height * menu_info[menu].num_items;

    if(debug)
	printf("creating menuwin x,y = %i,%i\n",x,y);

    /* create opaque window */
    
    menuwin = XCreateSimpleWindow(ms_display, RootWindow(ms_display, 
            ms_screen), x, y, menu_width, menu_height, 
            border_width, ms_foreground, ms_background);
            
    /* make it a menu window */
    
    valuemask = CWOverrideRedirect | CWSaveUnder;
    attributes.save_under = True;
    attributes.override_redirect = True;
    
    XChangeWindowAttributes(ms_display, menuwin, valuemask, &attributes);
    
    if(debug)
	printf("creating panes \n");


    /* create the choice windows for the text */
    for (winindex = 0; winindex < menu_info[menu].num_items; winindex++) {
        panes[winindex] = XCreateSimpleWindow(ms_display, menuwin, 0, 
                menu_height/menu_info[menu].num_items*winindex, menu_width, 
                pane_height, border_width = 1, 
                ms_foreground, ms_background);
        XSelectInput(ms_display, panes[winindex], ButtonPressMask
                | ButtonReleaseMask |  ExposureMask | EnterWindowMask | LeaveWindowMask);
    }

    XSelectInput(ms_display, RootWindow(ms_display, ms_screen), 
            SubstructureNotifyMask);
            
    if(debug)
	printf("mapping subs \n");


    /* these don't appear until parent (menuwin) is mapped */
    XMapSubwindows(ms_display,menuwin);

    /* create the cursor for the menu */
    cursor = XCreateFontCursor(ms_display, XC_left_ptr);
    hand_cursor = XCreateFontCursor(ms_display, XC_hand2);

    XDefineCursor(ms_display, menuwin, cursor);

    /* create GC's for regular, reversed and grayed items */
    gc = XCreateGC(ms_display, RootWindow(ms_display, ms_screen) , 0, NULL);
    XSetForeground(ms_display, gc, ms_foreground);
    XSetFont(ms_display,gc,ms_font_info->fid);

    rgc = XCreateGC(ms_display, RootWindow(ms_display, ms_screen) , 0, NULL);
    XSetForeground(ms_display, rgc, ms_background);
    XSetFont(ms_display,rgc,ms_font_info->fid);
    /* grayed out gc */
    ggc = XCreateGC(ms_display, RootWindow(ms_display, ms_screen) , 0, NULL);
    XSetFont(ms_display,ggc,ms_font_info->fid);
    if(XLookupColor(ms_display,ms_colormap,DISABLED_COLOR,&color_exact,&color_screen) != 0)
    {
        	XAllocColor(ms_display,ms_colormap,&color_screen);
    
    		XSetForeground(ms_display, ggc, color_screen.pixel);
        	
        	if(debug)
        		printf("menusub: XlookupColor ok \n");
    }
    else
    	XSetForeground(ms_display,ggc,ms_background); /* default to invisible disabled items */
    

    if(debug)
	printf("mapping menuwin \n");


    /* map the menu window (and its subwindows) to the screen_num */
    XMapWindow(ms_display, menuwin);

            /* all events are sent to the grabbing
             * window regardless of
             * whether this is True or False. 
             * owner_events only affects the distribution of
             * events when the pointer is within this
             * application's windows. */
            owner_events = True;    

            /* we don't want pointer or keyboard events
             * frozen in the server */
            pointer_mode = GrabModeAsync;
            keyboard_mode = GrabModeAsync;

            /* we don't want to confine the cursor */

            confine_to = None;
            
            if(debug)
		printf("grabbing pointer \n");

            XGrabPointer(ms_display, menuwin, owner_events, 
                    ButtonPressMask | ButtonReleaseMask, 
                    pointer_mode, keyboard_mode, 
                    confine_to, hand_cursor, CurrentTime);

 	if(debug)
		printf("starting event loop \n");
		

	/* we start with no panes inverted */
			
	inverted_pane = -1;

                   
    /* loop getting events on the menu window and icons */
    while (1)
        {
        /* wait for an event */
        XNextEvent(ms_display, &event);

        /* if expose, draw text in pane if it is pane */
        switch (event.type) {
        case Expose:

                if (inverted_pane == event.xexpose.window)
                    paint_pane(event.xexpose.window, panes, gc, rgc, ggc, BLACK,menu);
                else
                    paint_pane(event.xexpose.window, panes, gc, rgc, ggc, WHITE,menu);

            break;
            
        case EnterNotify:
        	paint_pane(event.xcrossing.window,panes,gc,rgc,ggc,BLACK,menu);
        	inverted_pane = event.xcrossing.window;
        break;
        
        case LeaveNotify:
        	paint_pane(event.xcrossing.window,panes,gc,rgc,ggc,WHITE,menu); 
        break;
         
        case ButtonRelease:
        
                /* convert window ID to 
                 * window array index  */
                winindex = -1;
                for (i = 0; i < menu_info[menu].num_items; i++)
                {
                	if(event.xbutton.window == panes[i])
                		winindex = i;
                } 
                if(debug)
                	printf(" index = %i \n",winindex);
 

                XSetInputFocus(ms_display, 
                        RootWindow(ms_display,ms_screen), 
                        RevertToPointerRoot, 
                        CurrentTime);
                    /* need to change focus border
                     * width back here */
                 XDestroyWindow(ms_display, menuwin);
              	 XFreeGC(ms_display, gc);
                 XFreeGC(ms_display, rgc);
                 XFreeGC(ms_display, ggc);
                 XFlush(ms_display);
                 
                 if(menu_info[menu].disable[winindex] == NO) /* item enabled */
    		 {
                 	return(winindex);
                 }
                 else
                 	return(-1);
                   
            break;
            
        } /* end switch */
    } /* end menu loop (while) */
} /* end main */

paint_pane(window, panes, ngc, rgc, ggc, mode, menu)
Window window;
Window panes[];
GC ngc,rgc,ggc;
int mode,menu;
{      
    int win,pwin;
    int x = H_BORDER, y;
    GC gc;
    
    /* find out index of window for label text */
    pwin = -1;
    for (win = 0; win < menu_info[menu].num_items ; win++)
    {
    	if(window == panes[win])
    		pwin = win;
    }
    if(pwin == -1)
       		return;

    if(menu_info[menu].disable[pwin] == NO) /* item enabled */
    {
    	if (mode == BLACK) {
        	XSetWindowBackground(ms_display, window, ms_foreground);
        	gc = rgc;
    	}
    	else {
        	XSetWindowBackground(ms_display, window, ms_background);
        	gc = ngc;
    	}
    }
    else /* grayed out */
    {
        	XSetWindowBackground(ms_display, window, ms_background);
        	gc = ggc;    
    }
    
    /* clearing repaints the background */
    XClearWindow(ms_display, window);


    y = ms_font_info->max_bounds.ascent + V_BORDER;

    /* the string length is necessary because strings for
        XDrawString may not be NULL terminated */
    XDrawString(ms_display, window, gc, x, y, menu_info[menu].label[pwin], 
            strlen( menu_info[menu].label[pwin])); 
}

